import requests
import json
import logging
import time

logger = logging.getLogger(__name__)

MEM_ALLOCATED_TARGET = "RBstat?target=circle.*.memory.allocated&format=json&time=-2min"
MEM_USAGE_TARGET = "RBstat?target=circle.*.memory.usage&format=json&time=-2min"
VMCOUNT_TARGET = "RBstat?target=circle.*.vmcount&format=json&time=-2min"
CPU_PERCENT_TARGET = "RBstat?target=circle.*.cpu.percent&format=json&time=-2min"
STATIC_INFO = "dashboard/acpi/nodesum/"

STAT_TARGETS = {
    "mem_allocated": MEM_ALLOCATED_TARGET,
    "mem_usage": MEM_USAGE_TARGET,
    "vmcount": VMCOUNT_TARGET,
    "cpu_percent": CPU_PERCENT_TARGET,
}


def get_static_info_from_dc(datacenter):
    url: str = f"{datacenter}/{STATIC_INFO}"
    req = requests.request(
        method="GET",
        url=url,
        verify=False,
    )
    if int(req.status_code / 100) != 2:
        logger.warning(
            f"cannot retrieve the data, from dc: {datacenter}  ({req.status_code})"
        )
        return 0, 0
    else:
        try:
            data = json.loads(req.content)
            core_num = data["core_num"]
            max_ram = data["max_ram"]
            return core_num, max_ram
        except json.decoder.JSONDecodeError:
            logger.warning("JSON decoding error: Static infos")
            return 0, 0


def get_stat_from_dc(datacenter, retry=False):
    stat = {}
    for key, target in STAT_TARGETS.items():
        url: str = f"{datacenter}/{target}"
        req = requests.request(
            method="GET",
            url=url,
            verify=False,
        )
        if int(req.status_code / 100) != 2:
            logger.warning(
                f"cannot retrieve the data, from dc: {datacenter}  ({req.status_code})"
            )
        else:
            try:
                data = json.loads(req.content)
                stat[key] = [
                    max(
                        i["datapoints"][0][0] if i["datapoints"][0][0] else 0.0,
                        i["datapoints"][1][0] if i["datapoints"][1][0] else 0.0,
                    )
                    for i in data
                ]

                if retry and stat[key][0] is None:
                    time.sleep(1)
                    req = requests.request(
                        method="GET",
                        url=url,
                        verify=False,
                    )
                    data = json.loads(req.content)
                    stat[key] = [
                        max(
                            i["datapoints"][0][0] if i["datapoints"][0][0] else 0.0,
                            i["datapoints"][1][0] if i["datapoints"][1][0] else 0.0,
                        )
                        for i in data
                    ]
            except json.decoder.JSONDecodeError:
                logger.warning(f"JSON decoding error: key {key}")
                stat[key] = 0.0
    return stat
