from fastapi.responses import ORJSONResponse
from sredis.sredis import (
    available_ram,
    get_all_datacenter,
    get_datacenter_token,
    pop_vm,
    rr_get,
    save_rpvm,
)
from fastapi import HTTPException
import requests
import json
from core.models import ResourcePoolSchema
import logging

logger = logging.getLogger(__name__)


def proxy_datacenters(
    serverpath: str,
    username,
    method="GET",
    body="",
    balancer_fun=rr_get,
    datacenter=None,
):
    vm_ram = 4096
    cpu_core = 1
    if "ram_size" in body:
        vm_ram = body["ram_size"]
    if "num_cores" in body:
        cpu_core = body["num_cores"]
    if "datacenter" in body:
        server = body["datacenter"]
    elif datacenter:
        server = datacenter
    else:
        server = balancer_fun(vm_ram, cpu_core)
    token = get_datacenter_token(username, server)
    url = f"{server}/{serverpath}"
    try:
        t_resp = requests.request(
            json=body,
            method=method,
            url=url,
            allow_redirects=True,
            verify=False,
            headers={"Authorization": token},
        )
        if int(t_resp.status_code / 100) != 2:
            raise HTTPException(
                status_code=t_resp.status_code, detail="Remote server error"
            )
        if not t_resp.content:
            return ORJSONResponse({"status": "ok"}, status_code=t_resp.status_code)
        obj = json.loads(t_resp.content)
        if type(obj) is list:
            for o in obj:
                o["datacenter"] = server
        else:
            obj["datacenter"] = server
        response = ORJSONResponse(obj, status_code=t_resp.status_code)
        return response
    except Exception:
        raise HTTPException(status_code=500, detail="Remote server error")


def create_vm_from_template(
    datacenter: str, username: str, name: str, template_name: str
):
    token = get_datacenter_token(username, datacenter)
    body = {"template_name": template_name, "username": username, "name": name}
    try:
        t_resp = requests.request(
            json=body,
            method="POST",
            url=f"{datacenter}dashboard/acpi/bvm/",
            allow_redirects=True,
            verify=False,
            headers={"Authorization": token},
        )
        if int(t_resp.status_code / 100) != 2:
            raise HTTPException(
                status_code=t_resp.status_code, detail="Remote server error"
            )
        obj = json.loads(t_resp.content)
        obj["datacenter"] = datacenter
        return obj
    except Exception:
        raise HTTPException(status_code=500, detail="Remote server error")


def create_rp(username: str, rp: ResourcePoolSchema):
    id = 0
    errors = 0
    if rp.manual:
        for dc, cnt in rp.manual.items():
            for _ in range(cnt):
                try:
                    create_vm_from_template(
                        dc,
                        username,
                        name=f"{rp.rpname} #{id}",
                        template_name=rp.from_template,
                    )

                    save_rpvm(rp.rpname, username, dc, id, rp.key)

                    id += 1
                except Exception:
                    errors += 1
    else:
        details = check_template_details_from_name(rp.from_template, username)
        ram_sum = details["ram_size"] * rp.num_vms
        avail_ram = available_ram()
        avail_ram_sum = sum(avail_ram.values())
        frac = ram_sum / avail_ram_sum
        for dc, ram in avail_ram.items():
            cnt = int(frac * ram / details["ram_size"])
            for _ in range(cnt):
                try:
                    create_vm_from_template(
                        dc,
                        username,
                        name=f"{rp.rpname} #{id}",
                        template_name=rp.from_template,
                    )

                    save_rpvm(rp.rpname, username, dc, id, rp.key)

                    id += 1
                except Exception:
                    errors += 1

    return id, errors


def check_template_details_from_name(template_name, username):
    dcs = get_all_datacenter()

    details = {"ram_size": 0, "num_cores": 1}

    for dc in dcs:
        resp = proxy_datacenters(
            f"dashboard/acpi/template?name={template_name}", username, datacenter=dc
        )
        if resp.status_code != 200:
            raise HTTPException(
                tatus_code=500,
                detail=f"Remote server error: {dc} - code {resp.status_code}",
            )
        details["num_cores"] = max(details["num_cores"], resp["num_cores"])
        details["ram_size"] = max(details["ram_size"], resp["ram_size"])

    return details


def update_vm_owner(datacenter, vm_id, username):
    pass


def get_vm_from_rp(username, rpname, key):
    vm = pop_vm(rpname, key)
    if vm is None:
        raise HTTPException(status_code=400, detail="Bad resource pool or key")
    datacenter, id = vm
    update_vm_owner(datacenter, id, username)
    token = get_datacenter_token(username, datacenter)
    try:
        t_resp = requests.request(
            method="GET",
            url=f"{datacenter}dashboard/acpi/vm/{id}",
            allow_redirects=True,
            verify=False,
            headers={"Authorization": token},
        )
        if int(t_resp.status_code / 100) != 2:
            raise HTTPException(
                status_code=t_resp.status_code, detail="Remote server error"
            )
        obj = json.loads(t_resp.content)
        obj["datacenter"] = datacenter
        return obj
    except Exception:
        raise HTTPException(status_code=500, detail="Remote server error")
