from fastapi import FastAPI, Response, Body, Depends
import os
from typing import List
from balancer.util import get_vm_from_rp, proxy_datacenters, create_rp
from sredis.sredis import (
    check_user,
    create_puser,
    add_datacenter,
    get_all_stats,
    set_token,
    create_superpuser,
    get_rtokens,
    reset,
    update_datacenter_stats,
    update_datacenter_status,
    update_stats,
    valid_ram_datacenters,
)
import logging
import requests
from fastapi_utils.tasks import repeat_every
from core.models import (
    User,
    Token,
    UserLoginSchema,
    DataCenterSchema,
    ResourcePoolSchema,
    ResourceStat,
    VMFromRPSchema,
)
from core.auth import signJWT
from core.bearer import get_current_user, admin_user
from redis_om import Migrator

logging.config.fileConfig("logging.conf", disable_existing_loggers=False)

# get root logger
logger = logging.getLogger(__name__)
requests.packages.urllib3.disable_warnings()
Migrator().run()

app = FastAPI()

reset()

add_datacenter("https://kappa1.fured.cloud.bme.hu")
add_datacenter("https://kappa2.fured.cloud.bme.hu")
add_datacenter("https://kappa3.fured.cloud.bme.hu")

create_superpuser(
    User(username="admin", email="example@domain.com", password=os.getenv("ADMIN_PASS"))
)


@app.post("/user/signup", tags=["user"])
async def create_user(user: User = Body(...)):
    create_puser(user)
    return signJWT(user.username)


@app.post("/user/login", tags=["user"])
async def user_login(user: UserLoginSchema = Body(...)):
    if check_user(user):
        return signJWT(user.username)
    return {"error": "Wrong login details!"}


@app.get("/lb/{server_path:path}")
def proxy_get(
    server_path: str = "/", username=Depends(get_current_user), datacenter: str = ""
):
    return proxy_datacenters(server_path, username, body="", datacenter=datacenter)


@app.delete("/lb/{server_path:path}")
def proxy_delete(
    server_path: str = "/", username=Depends(get_current_user), datacenter: str = ""
):
    return proxy_datacenters(
        server_path, username, method="DELETE", datacenter=datacenter
    )


@app.post("/lb/rr/{server_path:path}")
def proxy_post_rr(
    server_path: str = "/", username=Depends(get_current_user), body=Body()
):
    return proxy_datacenters(server_path, username, body=body, method="POST")


@app.post("/lb/ff/{server_path:path}")
def proxy_post_ff(
    server_path: str = "/", username=Depends(get_current_user), body=Body()
):
    return proxy_datacenters(server_path, username, body=body, method="POST")


@app.post("/createrp/")
def create_resource_pool(
    username=Depends(get_current_user), rp: ResourcePoolSchema = Body(...)
):
    create_rp(username, rp)


@app.post("/rp/")
def get_vm_from_pool(
    username=Depends(get_current_user), vm: VMFromRPSchema = Body(...)
):
    return get_vm_from_rp(username, vm.rpname, vm.key)


@app.post("/add_datacenter/")
def create_datacenter(dc: DataCenterSchema = None, username=Depends(admin_user)):
    add_datacenter(dc.url)
    return Response(status_code=201)


@app.post("/set_tokens/")
def set_tokens(tokens: List[Token] = None, username=Depends(get_current_user)):
    for token in tokens:
        set_token(username, str(token.datacenter), str(token.token))
    return tokens


@app.post("/updatestat/")
def update_datacenter_stat(
    stats: List[ResourceStat] = None, username=Depends(admin_user)
):
    for stat in stats:
        update_stats(stat)
    return stats


@app.get("/stats/")
def get_datacenter_stat(username=Depends(get_current_user)):
    stats = get_all_stats()
    stats = [
        ResourceStat(
            datacenter_name=x.datacenter_name,
            mem_max=x.mem_max,
            mem_used=x.mem_used,
            cpu_usage=x.cpu_usage,
            vm_count=x.vm_count,
            cpu_core=x.cpu_core,
        )
        for x in stats
    ]
    return stats


@app.get("/stats/ram/")
def get_datacenter_stat_ram(username=Depends(get_current_user), vm_ram: int = 1024):
    stats = valid_ram_datacenters(vm_ram)
    return stats


@app.get("/tokens/")
def get_tokens(username=Depends(get_current_user)):
    return get_rtokens(username=username)


@app.on_event("startup")
@repeat_every(seconds=300, raise_exceptions=True)
def update_datacenter_healt():
    update_datacenter_status()


@app.on_event("startup")
@repeat_every(seconds=200, raise_exceptions=True)
def update_datacenter_stats_rep():
    update_datacenter_stats()
