import redis
import requests
from balancer.stats import get_stat_from_dc, get_static_info_from_dc
from core.models import UserLoginSchema, ResourceStat
from .models import PUser, DataCenterResource, ResourcePool
from passlib.hash import pbkdf2_sha256
from core.models import User, DataCenterSchema
from core.auth import hash_pass
from fastapi import HTTPException
import random
import logging

logger = logging.getLogger(__name__)


def get_status(datacenter_url: str):
    logger.info(f"Health check datacenter '{datacenter_url}'")
    try:
        resp = requests.request(method="GET", url=datacenter_url, verify=False)
    except Exception:
        return False
    return int(resp.status_code / 100) == 2


r = redis.Redis(host="localhost", port=6379, db=0, decode_responses=True)


def reset():
    logging.info("Clean settings")
    r.set("datacenters_cnt", 1)
    all_keys = r.hgetall("datacenters_hash")
    if all_keys.keys():
        r.hdel("datacenters_hash", *list(all_keys.keys()))

    r.delete("rrlist")
    pipe = r.pipeline()
    for dc in all_keys.values():
        pipe.rpush("rrlist", dc)
    pipe.execute()


def get_all_datacenter():
    return r.lrange("rrlist", 0, -1)


def update_status(datacenter):
    status = "OK" if get_status(datacenter_url=datacenter) is True else "Wrong"
    if status == "Wrong":
        logger.warning(f"{datacenter} is inactive status")
    r.hset("datacenter_status", datacenter, status)


def update_datacenter_status():
    logger.info("Update datacenters status")
    all_keys = r.hgetall("datacenters_hash")
    for dc in all_keys.values():
        update_status(datacenter=dc)


def update_datacenter_stats():
    logger.info("Update datacenters stats")
    all_keys = r.hgetall("datacenters_hash")
    for dc in all_keys.values():
        logger.info(f"Get stats from {dc}")
        stats = get_stat_from_dc(dc, retry=True)
        cpu_core, mem_max = get_static_info_from_dc(dc)
        rs = ResourceStat(
            datacenter_name=dc,
            cpu_core=cpu_core,
            mem_max=int(mem_max / 1048576) if mem_max else 4096,
            mem_allocated=int(stats["mem_allocated"][0] / 1048576.0)
            if stats["mem_allocated"]
            else 0,
            mem_used=int(stats["mem_usage"][0] * 4096 / 100.0)
            if stats["mem_usage"][0]
            else 0,
            cpu_usage=float(stats["cpu_percent"][0])
            if stats["cpu_percent"][0]
            else 0.0,
            vm_count=int(stats["vmcount"][0]) if stats["vmcount"][0] else 0,
        )
        update_stats(rs)


def add_datacenter(datacenter: str):
    cnt = int(r.get("datacenters_cnt"))
    r.hset("datacenters_hash", cnt, datacenter)
    r.incr("datacenters_cnt")
    r.set("roundrobin_cnt", 1)
    r.rpush("rrlist", datacenter)
    update_status(datacenter)


def rr_get(vm_ram, cpu_core):
    rr = r.rpoplpush("rrlist", "rrlist")
    valid = valid_ram_datacenter(rr, vm_ram)
    while not valid:
        rr = r.rpoplpush("rrlist", "rrlist")
        valid = valid_ram_datacenter(rr, vm_ram)
    return rr


def wr_get(centers: DataCenterSchema):
    cnt = int(r.get("datacenters_cnt"))
    indexis = range(1, cnt + 1)
    return random.choice(indexis, weights=())[0]


def available_ram():
    dcs = DataCenterResource.find().all()
    valid_dcs = {}
    for dc in dcs:
        valid_dcs[dc.datacenter_name] = dc.mem_max - dc.mem_used
    return valid_dcs


def set_token(username: str, datacenter: str, token: str):
    r.hset(f"tokens:{username}", datacenter, token)


def get_rtoken(username: str, datacenter: str):
    return r.hget(f"tokens:{username}", datacenter)


def get_rtokens(username: str):
    center_ids = list(r.hgetall("datacenters_hash").keys())
    tokens = []
    for id in center_ids:
        token = {}
        token["datacenter"] = r.hget("datacenters_hash", id)
        token["token"] = get_rtoken(username=username, datacenter=token["datacenter"])
        tokens.insert(0, token)
    return tokens


def get_datacenter_token(username: str, datacenter: str):
    return str(r.hget(f"tokens:{username}", datacenter))


def check_user(data: UserLoginSchema):
    user = PUser.find(PUser.username == data.username).all()
    if pbkdf2_sha256.verify(data.password, user[0].password):
        return user[0]
    return False


def create_puser(user: User):
    s = PUser.find(PUser.username == user.username).all()
    if s:
        raise HTTPException(status_code=403, detail="User already exists")
    user = PUser(
        username=user.username, email=user.email, password=hash_pass(user.password)
    )
    user.save()


def create_superpuser(user: User):
    s = PUser.find(PUser.username == user.username).all()
    if not s:
        puser = PUser(
            username=user.username,
            email=user.email,
            password=hash_pass(user.password),
            admin=1,
        )
        puser.save()
        return True
    return False


def update_stats(stat: ResourceStat):
    dc = DataCenterResource.find(
        DataCenterResource.datacenter_name == stat.datacenter_name
    ).all()
    if not dc or len(dc) == 0:
        dc = DataCenterResource(
            datacenter_name=stat.datacenter_name,
            mem_max=stat.mem_max,
            mem_used=stat.mem_used,
            cpu_usage=stat.cpu_usage,
            vm_count=stat.vm_count,
            cpu_core=stat.cpu_core,
        )
        dc.save()
    else:
        dc = dc[0]
        dc.mem_max = stat.mem_max if stat.mem_max else dc.mem_max
        dc.mem_used = stat.mem_used if stat.mem_used else dc.mem_used
        dc.cpu_usage = stat.cpu_usage if stat.cpu_usage else dc.cpu_usage
        dc.vm_count = stat.vm_count if stat.vm_count else dc.vm_count
        dc.cpu_core = stat.cpu_core if stat.cpu_core else dc.cpu_core
        dc.save()


def get_stats(datacenter_name: str):
    dc = DataCenterResource.find(
        DataCenterResource.datacenter_name == datacenter_name
    ).get_item()
    return dc


def valid_ram_datacenter(datacenter_name: str, vm_mem: int):
    dc = DataCenterResource.find(
        DataCenterResource.datacenter_name == datacenter_name
    ).get_item()
    if dc is None:
        return False
    if dc.mem_max - dc.mem_used > vm_mem:
        return True
    return False


def valid_ram_datacenters(vm_mem: int):
    dcs = DataCenterResource.find().all()
    valid_dcs = []
    for dc in dcs:
        if dc.mem_max - dc.mem_used > vm_mem:
            valid_dcs.append(dc.datacenter_name)
    return valid_dcs


def get_all_stats():
    dcs = DataCenterResource.find().all()
    return dcs


def save_rpvm(rpname, username, datacenter, id, token):
    rpvm = ResourcePool(
        rpname=rpname, owner=username, datacenter=datacenter, id=id, token=token
    )
    rpvm.save()


def pop_vm(rpname, key):
    rpvm = ResourcePool.find(ResourcePool.rpname == rpname).first()
    if rpvm.token == key:
        vm = rpvm.datacenter, rpvm.id
        rpvm.delete()
        return vm
    return None
